package com.deepexi.scd.engine.calculate.converter;

package ..deTemp.1736053371876;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.NumberUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.deepexi.scd.calculate.domain.dto.CalculateCarbonFootprintApiDTO;
import com.deepexi.scd.calculate.domain.dto.CalculateCarbonFootprintTableApiDTO;
import com.deepexi.scd.calculate.domain.dto.CalculateTemplateEdgeApiDTO;
import com.deepexi.scd.calculate.domain.dto.CalculateTemplateFieldApiDTO;
import com.deepexi.scd.calculate.domain.dto.CalculateTemplateFieldItemApiDTO;
import com.deepexi.scd.calculate.domain.dto.CalculateTemplateItemApiDTO;
import com.deepexi.scd.calculate.domain.dto.CalculateTemplateItemValueApiDTO;
import com.deepexi.scd.calculate.domain.dto.CalculateTemplateMetaApiDTO;
import com.deepexi.scd.calculate.domain.dto.CalculateTemplateTableCalculateApiDTO;
import com.deepexi.scd.calculate.domain.dto.CommonDdictMemberApiDTO;
import com.deepexi.scd.calculate.domain.query.CalculateTemplateEdgeApiQuery;
import com.deepexi.scd.calculate.domain.query.CalculateTemplateFieldItemApiQuery;
import com.deepexi.scd.calculate.domain.query.CalculateTemplateItemApiQuery;
import com.deepexi.scd.calculate.domain.query.CalculateTemplateMetaApiQuery;
import com.deepexi.scd.calculate.domain.query.CalculateTemplateTableCalculateApiQuery;
import com.deepexi.scd.calculate.domain.query.CommonDdictMemberApiQuery;
import com.deepexi.scd.common.enums.BaseExceptionEnum;
import com.deepexi.scd.common.enums.FootprintAggFieldTypeEnum;
import com.deepexi.scd.common.enums.business.CalculateItemValueScalarTypeEnum;
import com.deepexi.scd.common.enums.business.CalculateItemValueTypeEnum;
import com.deepexi.scd.common.enums.business.CalculateStandardFieldValueTypeEnum;
import com.deepexi.scd.common.enums.business.FillValueTypeEnum;
import com.deepexi.scd.common.util.BeanConverter;
import com.deepexi.scd.common.util.ThrowEnumUtils;
import com.deepexi.scd.component.fill.domain.request.FactorSelectComponentRequest;
import com.deepexi.scd.component.fill.domain.vo.FactorCalculateComponentVO;
import com.deepexi.scd.component.fill.remote.calculate.CalculateTemplateEdgeRemote;
import com.deepexi.scd.component.fill.remote.calculate.CalculateTemplateFieldItemRemote;
import com.deepexi.scd.component.fill.remote.calculate.CalculateTemplateItemRemote;
import com.deepexi.scd.component.fill.remote.calculate.CalculateTemplateMetaRemote;
import com.deepexi.scd.component.fill.remote.calculate.CalculateTemplateTableCalculateRemote;
import com.deepexi.scd.component.fill.remote.calculate.CommonDdictMemberRemote;
import com.deepexi.scd.component.fill.service.CarbonFootprintModelService;
import com.deepexi.scd.engine.calculate.converter.BaseParamConverter;
import com.deepexi.scd.engine.calculate.domain.CalculateStandardParamConvertResult;
import com.deepexi.scd.engine.calculate.domain.CarbonFootprintParamConvertResult;
import com.deepexi.scd.engine.calculate.domain.CarbonFootprintParamRequest;
import com.deepexi.scd.engine.calculate.domain.dto.CalculateItemValueEnumDTO;
import com.deepexi.scd.engine.calculate.enums.ResultEnum;
import com.deepexi.scd.engine.calculate.util.SpringContextHolder;
import com.deepexi.spa.commons.utils.CollectionUtils;
import com.deepexi.spa.commons.utils.StringUtils;
import com.google.api.client.util.Lists;
import com.google.api.client.util.Maps;
import com.google.common.base.Joiner;
import com.google.common.collect.ImmutableList;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Collection;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.function.Function;
import java.util.function.Supplier;
import java.util.stream.Collectors;
import java.util.stream.Stream;
import org.apache.commons.lang3.math.NumberUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class FootprintParamConverter extends BaseParamConverter<CarbonFootprintParamRequest, CarbonFootprintParamConvertResult> {
  private static final Logger log = LoggerFactory.getLogger(com.deepexi.scd.engine.calculate.converter.FootprintParamConverter.class);
  
  private List<String> energyCategoryList;
  
  private Long modelId;
  
  private Long modelVersion;
  
  private CalculateCarbonFootprintApiDTO footprint;
  
  private List<CalculateTemplateItemApiDTO> itemList;
  
  private Map<Long, List<CalculateTemplateFieldItemApiDTO>> fieldItemMap;
  
  private Map<String, Map<String, CalculateTemplateFieldApiDTO>> fieldCodeMap = new HashMap<>();
  
  private Map<String, Map<Long, List<CalculateTemplateTableCalculateApiDTO>>> tableCalculateMap = new HashMap<>();
  
  private Map<String, Map<Long, List<CalculateTemplateFieldApiDTO>>> fieldMap = new HashMap<>();
  
  private Map<String, Map<Long, CalculateCarbonFootprintTableApiDTO>> tableMap = new HashMap<>();
  
  private Map<String, Map<Long, CalculateTemplateMetaApiDTO>> metaMap = new HashMap<>();
  
  private Map<String, Map<Long, CalculateTemplateEdgeApiDTO>> edgeMap = new HashMap<>();
  
  private Map<String, String> factorEnumMap = new HashMap<>();
  
  private CarbonFootprintModelService carbonFootprintModelService;
  
  public void init(String modelIdStr, String versionLong) {
    Long version = null;
    Long modelId = Long.valueOf(Long.parseLong(modelIdStr));
    log.info("初始化编码：" + modelId);
    checkCode(modelIdStr);
    initTemplate(modelIdStr, version);
    initEdge();
    initMeta();
    initTable();
    initField();
    initFormula();
    initTemplate();
    initItem();
    initFieldItem();
    initEnergyCategory();
  }
  
  private void initTemplate(String modelIdStr, Long version) {
    this.modelId = Long.valueOf(Long.parseLong(modelIdStr));
    this.modelVersion = version;
    CarbonFootprintModelService carbonFootprintModelService = (CarbonFootprintModelService)SpringContextHolder.getBean(CarbonFootprintModelService.class);
    this.footprint = carbonFootprintModelService.getById(this.modelId);
    ThrowEnumUtils.isNull(this.footprint).throwMessage((BaseExceptionEnum)ResultEnum.FOOTPRINT_CODE_VERSION_NOT_EXIST, new Object[] { this.modelId, version });
    ThrowEnumUtils.isEmpty(this.footprint.getTableList()).throwMessage((BaseExceptionEnum)ResultEnum.FOOTPRINT_TABLE_CANNOT_EMPTY, new Object[] { this.modelId, version });
  }
  
  private void initTemplate() {}
  
  private void initEdge() {
    List<CalculateCarbonFootprintTableApiDTO> tableList = this.footprint.getTableList();
    List<Long> edgeIdList = (List<Long>)tableList.stream().map(item -> item.getEdgeId()).distinct().collect(Collectors.toList());
    CalculateTemplateEdgeRemote calculateTemplateEdgeRemote = (CalculateTemplateEdgeRemote)SpringContextHolder.getBean(CalculateTemplateEdgeRemote.class);
    CalculateTemplateEdgeApiQuery calculateTemplateEdgeApiQuery = new CalculateTemplateEdgeApiQuery();
    calculateTemplateEdgeApiQuery.setEdgeIds(edgeIdList);
    Map<Long, CalculateTemplateEdgeApiDTO> standardEdgeMap = (Map<Long, CalculateTemplateEdgeApiDTO>)((List)Optional.<Object>ofNullable(calculateTemplateEdgeRemote.findList(calculateTemplateEdgeApiQuery).getPayload()).orElse(Lists.newArrayList())).stream().collect(Collectors.toMap(CalculateTemplateEdgeApiDTO::getEdgeId, Function.identity()));
    this.edgeMap.put("TEMPLATE", standardEdgeMap);
  }
  
  private void initMeta() {
    List<CalculateCarbonFootprintTableApiDTO> tableList = this.footprint.getTableList();
    List<Long> metaIdList = (List<Long>)tableList.stream().map(item -> item.getMetaId()).distinct().collect(Collectors.toList());
    CalculateTemplateMetaRemote calculateTemplateMetaRemote = (CalculateTemplateMetaRemote)SpringContextHolder.getBean(CalculateTemplateMetaRemote.class);
    CalculateTemplateMetaApiQuery metaApiQuery = new CalculateTemplateMetaApiQuery();
    metaApiQuery.setMetaIdList(metaIdList);
    Map<Long, CalculateTemplateMetaApiDTO> standardMetaMap = (Map<Long, CalculateTemplateMetaApiDTO>)((List)Optional.<Object>ofNullable(calculateTemplateMetaRemote.findList(metaApiQuery).getPayload()).orElse(Lists.newArrayList())).stream().collect(Collectors.toMap(CalculateTemplateMetaApiDTO::getMetaId, Function.identity()));
    this.metaMap.put("TEMPLATE", standardMetaMap);
  }
  
  private void initTable() {
    List<CalculateCarbonFootprintTableApiDTO> tableList = this.footprint.getTableList();
    Map<Long, CalculateCarbonFootprintTableApiDTO> standardTableMap = (Map<Long, CalculateCarbonFootprintTableApiDTO>)tableList.stream().collect(Collectors.toMap(CalculateCarbonFootprintTableApiDTO::getTableId, Function.identity()));
    this.tableMap.put("TEMPLATE", standardTableMap);
  }
  
  private void initFormula() {
    CalculateTemplateTableCalculateRemote formulaRemote = (CalculateTemplateTableCalculateRemote)SpringContextHolder.getBean(CalculateTemplateTableCalculateRemote.class);
    CalculateTemplateTableCalculateApiQuery formulaQuery = new CalculateTemplateTableCalculateApiQuery();
    List<CalculateTemplateTableCalculateApiDTO> list = (List<CalculateTemplateTableCalculateApiDTO>)this.footprint.getTableList().stream().flatMap(item -> item.getTableCalculateList().stream()).collect(Collectors.toList());
    replaceFormula(list, "TEMPLATE");
    Map<Long, List<CalculateTemplateTableCalculateApiDTO>> standardFormulaMap = (Map<Long, List<CalculateTemplateTableCalculateApiDTO>>)list.stream().collect(Collectors.groupingBy(CalculateTemplateTableCalculateApiDTO::getTableId));
    this.tableCalculateMap.put("TEMPLATE", standardFormulaMap);
  }
  
  private void replaceFormula(List<CalculateTemplateTableCalculateApiDTO> formulaList, String type) {
    formulaList.forEach(formula -> {
          List<CalculateTemplateFieldApiDTO> fields = (List<CalculateTemplateFieldApiDTO>)((Map)this.fieldMap.get(type)).get(formula.getTableId());
          Map<String, String> fieldNameMap = (Map<String, String>)fields.stream().sorted(()).collect(Collectors.toMap(CalculateTemplateFieldApiDTO::getFieldName, CalculateTemplateFieldApiDTO::getCode, (), java.util.LinkedHashMap::new));
          String calculateFormula = StringUtils.defaultString(formula.getCalculateFormula(), formula.getCalculateFormulaDisplay());
          ThrowEnumUtils.isBlank(calculateFormula).throwMessage((BaseExceptionEnum)ResultEnum.FOOTPRINT_FORMULA_CANNOT_NULL, new Object[] { formula.getTableId() });
          for (Map.Entry<String, String> entry : fieldNameMap.entrySet()) {
            String key = entry.getKey();
            String value = entry.getValue();
            calculateFormula = calculateFormula.replaceAll(key, "{" + value + "}");
          } 
          formula.setCalculateFormula(calculateFormula);
          log.debug("经过转换后的表达式是{} 表ID=", calculateFormula, formula.getTableId());
        });
  }
  
  private void initField() {
    List<CalculateTemplateFieldApiDTO> fieldList = (List<CalculateTemplateFieldApiDTO>)this.footprint.getTableList().stream().flatMap(item -> item.getFields().stream()).collect(Collectors.toList());
    this.fieldMap.put("TEMPLATE", (Map<Long, List<CalculateTemplateFieldApiDTO>>)fieldList.stream().collect(Collectors.groupingBy(CalculateTemplateFieldApiDTO::getTableId)));
    this.fieldCodeMap.put("TEMPLATE", (Map<String, CalculateTemplateFieldApiDTO>)fieldList.stream().collect(Collectors.toMap(CalculateTemplateFieldApiDTO::getCode, Function.identity(), (v1, v2) -> v2)));
  }
  
  private void initItem() {
    CalculateTemplateItemRemote itemRemote = (CalculateTemplateItemRemote)SpringContextHolder.getBean(CalculateTemplateItemRemote.class);
    CalculateTemplateItemApiQuery itemApiQuery = new CalculateTemplateItemApiQuery();
    itemApiQuery.setTableIdList(getTableIds());
    this
      .itemList = Optional.<Object>ofNullable(itemRemote.findListWithValues(itemApiQuery).getPayload()).orElse(Lists.newArrayList());
  }
  
  private void initFieldItem() {
    CalculateTemplateFieldItemRemote fieldItemRemote = (CalculateTemplateFieldItemRemote)SpringContextHolder.getBean(CalculateTemplateFieldItemRemote.class);
    CalculateTemplateFieldItemApiQuery fieldItemApiQuery = new CalculateTemplateFieldItemApiQuery();
    List<Long> templateFieldIds = (List<Long>)this.fieldMap.values().stream().flatMap(item -> item.values().stream()).flatMap(Collection::stream).filter(item -> "SELECT_TREE".equals(item.getFieldControl())).map(CalculateTemplateFieldApiDTO::getFieldId).distinct().collect(Collectors.toList());
    if (CollectionUtils.isEmpty(templateFieldIds)) {
      this.fieldItemMap = Maps.newHashMap();
      return;
    } 
    fieldItemApiQuery.setFieldIdList(templateFieldIds);
    List<CalculateTemplateFieldItemApiDTO> fieldItemList = Optional.<Object>ofNullable(fieldItemRemote.findList(fieldItemApiQuery).getPayload()).orElse(Lists.newArrayList());
    this.fieldItemMap = (Map<Long, List<CalculateTemplateFieldItemApiDTO>>)fieldItemList.stream().collect(Collectors.groupingBy(CalculateTemplateFieldItemApiDTO::getTableId));
  }
  
  private List<Long> getTableIds() {
    List<Long> tableIds = Lists.newArrayList();
    tableIds.addAll((Collection<? extends Long>)((Map)this.tableMap.get("TEMPLATE")).values().stream()
        .map(i -> i.getTableId())
        .distinct().collect(Collectors.toList()));
    tableIds.addAll((Collection<? extends Long>)((Map)Optional.<Map>ofNullable(this.tableMap.get("TEMPLATE"))
        .orElse(Maps.newHashMap())).values().stream()
        .map(i -> i.getTableId()).distinct().collect(Collectors.toList()));
    return (List<Long>)tableIds.stream().distinct().collect(Collectors.toList());
  }
  
  private void initEnergyCategory() {
    CommonDdictMemberRemote commonDdictMemberRemote = (CommonDdictMemberRemote)SpringContextHolder.getBean(CommonDdictMemberRemote.class);
    CommonDdictMemberApiQuery memberApiQuery = new CommonDdictMemberApiQuery();
    memberApiQuery.setDdictTypes(ImmutableList.of("ENERGY_CATEGORY", "STANDARD_FACTOR_ENUM"));
    List<CommonDdictMemberApiDTO> list = Optional.<Object>ofNullable(commonDdictMemberRemote.findList(memberApiQuery).getPayload()).orElse(Lists.newArrayList());
    this.energyCategoryList = (List<String>)list.stream().filter(item -> "ENERGY_CATEGORY".equals(item.getDdictType())).map(CommonDdictMemberApiDTO::getDdictMember).collect(Collectors.toList());
    this.factorEnumMap = (Map<String, String>)list.stream().filter(item -> "STANDARD_FACTOR_ENUM".equals(item.getDdictType())).collect(Collectors.toMap(CommonDdictMemberApiDTO::getDdictMember, CommonDdictMemberApiDTO::getDdictMemberName));
  }
  
  public CarbonFootprintParamConvertResult convert(CarbonFootprintParamRequest data) {
    this.modelId = data.getModelId();
    CarbonFootprintParamConvertResult result = new CarbonFootprintParamConvertResult();
    List<CarbonFootprintParamConvertResult.CalculateStandardParam> paramList = Lists.newArrayList();
    paramList = (List<CarbonFootprintParamConvertResult.CalculateStandardParam>)data.getFootprintTableList().stream().map(table -> {
          CarbonFootprintParamConvertResult.CalculateStandardParam param = covertParam(CollectionUtil.emptyIfNull(table.getDataRowList()), table.getTableId(), "TEMPLATE");
          handleRelateField(param);
          checkCalculateItem(param);
          checkFactor(param);
          return param;
        }).collect(Collectors.toList());
    result.setParamList(paramList);
    result.setFootprintId(data.getFootprintId());
    result.setModelId(this.modelId);
    result.setModelVersion(this.modelVersion);
    result.setProductName(data.getProductName());
    result.setGroupCode(data.getGroupCode());
    result.setModelNumber(data.getModelNumber());
    return result;
  }
  
  private void handleRelateField(CarbonFootprintParamConvertResult.CalculateStandardParam param) {
    param.getItemList().forEach(item -> {
          Long tableId = item.getTableId();
        });
  }
  
  private void checkFactor(CarbonFootprintParamConvertResult.CalculateStandardParam param) {}
  
  private void checkCalculateItem(CarbonFootprintParamConvertResult.CalculateStandardParam param) {
    param.getItemList().forEach(item -> item.getTableId());
  }
  
  List<FactorCalculateComponentVO.FactorCalculateFieldItem> getCalItemComponentList(Long tableId) {
    FactorSelectComponentRequest request = new FactorSelectComponentRequest();
    request.setCalculateType(CalculateItemValueTypeEnum.CALCULATE_ITEM);
    request.setTableId(this.footprint.getId());
    request.setTableId(tableId);
    return this.carbonFootprintModelService.getFactorSelectComponentItems(this.footprint, request);
  }
  
  private void checkMajorTable(List<CalculateStandardParamConvertResult.CalculateStandardParam> params) {
    List<CalculateStandardParamConvertResult.CalculateStandardParam> noTargetTableList = (List<CalculateStandardParamConvertResult.CalculateStandardParam>)params.stream().filter(param -> (Objects.nonNull(param.getEdgeId()) && StringUtils.isBlank(param.getTableTargetName()))).collect(Collectors.toList());
    checkMajorTableDetail(noTargetTableList);
    Map<String, List<CalculateStandardParamConvertResult.CalculateStandardParam>> paramMap = (Map<String, List<CalculateStandardParamConvertResult.CalculateStandardParam>>)params.stream().filter(param -> (Objects.nonNull(param.getEdgeId()) && StringUtils.isNotBlank(param.getTableTargetName()))).collect(Collectors.groupingBy(CalculateStandardParamConvertResult.CalculateStandardParam::getTableTargetName));
    paramMap.forEach((key, values) -> checkMajorTableDetail(values));
  }
  
  private void checkMajorTableDetail(List<CalculateStandardParamConvertResult.CalculateStandardParam> params) {
    List<CalculateStandardParamConvertResult.CalculateStandardParam> relateTableList = (List<CalculateStandardParamConvertResult.CalculateStandardParam>)params.stream().filter(item -> Objects.nonNull(item.getRelateTableId())).collect(Collectors.toList());
    if (CollectionUtils.isNotEmpty(relateTableList)) {
      ((Map)relateTableList.stream().collect(Collectors.groupingBy(CalculateStandardParamConvertResult.CalculateStandardParam::getRelateTableId))).forEach((key, values) -> {
            List<String> tableNames = (List<String>)values.stream().map(CalculateStandardParamConvertResult.CalculateStandardParam::getTableName).collect(Collectors.toList());
            ThrowEnumUtils.isTure((values.size() > 1)).throwMessage((BaseExceptionEnum)ResultEnum.STANDARD_TABLES_CANT_FILL_ON_ONCE, new Object[] { Joiner.on("、").join((Iterable)tableNames) });
          });
      Map<Long, CalculateStandardParamConvertResult.CalculateStandardParam> targetTableMap = (Map<Long, CalculateStandardParamConvertResult.CalculateStandardParam>)params.stream().collect(Collectors.toMap(CalculateStandardParamConvertResult.CalculateStandardParam::getTableId, Function.identity(), (a, b) -> a));
      relateTableList.forEach(item -> {
            if (Objects.nonNull(targetTableMap.get(item.getRelateTableId())))
              ThrowEnumUtils.isTure(true).throwMessage((BaseExceptionEnum)ResultEnum.STANDARD_TABLES_CANT_FILL_ON_ONCE, new Object[] { ((CalculateStandardParamConvertResult.CalculateStandardParam)targetTableMap.get(item.getRelateTableId())).getTableName() + "、" + item.getTableName() }); 
          });
    } 
  }
  
  private void handleAppendTable(List<CalculateStandardParamConvertResult.CalculateStandardParam> params) {
    List<CalculateStandardParamConvertResult.CalculateStandardParam> appendTableParams = (List<CalculateStandardParamConvertResult.CalculateStandardParam>)params.stream().filter(item -> Objects.isNull(item.getEdgeId())).collect(Collectors.toList());
    appendTableParams.forEach(param -> {
          List<CalculateStandardParamConvertResult.CalculateDataFillItem> itemList = param.getItemList();
          CalculateCarbonFootprintTableApiDTO calculateTemplateTableApiDTO = (CalculateCarbonFootprintTableApiDTO)((Map)this.tableMap.get("TEMPLATE")).get(param.getTableId());
          List<CalculateTemplateFieldApiDTO> fields = (List<CalculateTemplateFieldApiDTO>)((Map)this.fieldMap.get("TEMPLATE")).get(calculateTemplateTableApiDTO.getTableId());
          Optional<CalculateTemplateFieldApiDTO> fixFieldOptional = fields.stream().filter(()).findFirst();
          if (!fixFieldOptional.isPresent())
            ThrowEnumUtils.isTure(Boolean.TRUE.booleanValue()).throwMessage((BaseExceptionEnum)ResultEnum.STANDARD_TABLE_ONLY_DEPENDENCE, new Object[] { param.getTableName() }); 
          CalculateTemplateFieldApiDTO calculateTemplateFieldApiDTO = fixFieldOptional.get();
          Map<String, Boolean> fixFieldValueMap = Maps.newHashMap();
          itemList.forEach(());
        });
  }
  
  private void checkTargetNameRepeat(List<CalculateStandardParamConvertResult.CalculateStandardParam> params) {
    Map<String, List<CalculateStandardParamConvertResult.CalculateStandardParam>> targetNameMap = (Map<String, List<CalculateStandardParamConvertResult.CalculateStandardParam>>)params.stream().filter(param -> StringUtils.isNotBlank(param.getTableTargetName())).collect(Collectors.groupingBy(param -> param.getTableId() + "-" + param.getTableTargetName()));
    targetNameMap.forEach((key, values) -> ThrowEnumUtils.isTure((values.size() > 1)).throwMessage((BaseExceptionEnum)ResultEnum.STANDARD_TABLE_TARGET_NAME_REPEAT, new Object[0]));
  }
  
  private CarbonFootprintParamConvertResult.CalculateStandardParam covertParam(List<CarbonFootprintParamRequest.CalculateDataRow> dataRowList, Long tableId, String type) {
    List<CarbonFootprintParamConvertResult.CalculateDataFillItem> calculateDataFillItems = handleItem(dataRowList, tableId, type);
    List<Long> tableIdList = (List<Long>)calculateDataFillItems.stream().map(CarbonFootprintParamConvertResult.CalculateDataFillItem::getTableId).distinct().collect(Collectors.toList());
    long count = tableIdList.size();
    ThrowEnumUtils.isTure((count > 1L)).throwMessage((BaseExceptionEnum)ResultEnum.STANDARD_ROW_DATA_NOT_SAME_TABLE, new Object[0]);
    CalculateCarbonFootprintTableApiDTO calculateTemplateTableApiDTO = (CalculateCarbonFootprintTableApiDTO)((Map)this.tableMap.get(type)).get(tableId);
    CarbonFootprintParamConvertResult.CalculateStandardParam param = getCommonParam(calculateTemplateTableApiDTO, type);
    param.setItemList(calculateDataFillItems);
    Map<String, List<CarbonFootprintParamConvertResult.CalculateDataFillItem>> itemMap = (Map<String, List<CarbonFootprintParamConvertResult.CalculateDataFillItem>>)calculateDataFillItems.stream().filter(item -> StringUtils.isNotBlank(item.getItemName())).collect(Collectors.groupingBy(CarbonFootprintParamConvertResult.CalculateDataFillItem::getItemName));
    itemMap.forEach((key, values) -> ThrowEnumUtils.isTure((values.size() > 1)).throwMessage((BaseExceptionEnum)ResultEnum.STANDARD_TABLE_ITEM_NAME_REPEAT, new Object[] { ((CarbonFootprintParamConvertResult.CalculateDataFillItem)values.get(0)).getItemName() }));
    return param;
  }
  
  private CarbonFootprintParamConvertResult.CalculateStandardParam getCommonParam(CalculateCarbonFootprintTableApiDTO calculateTemplateTableApiDTO, String type) {
    Long tableId = calculateTemplateTableApiDTO.getTableId();
    List<CalculateTemplateTableCalculateApiDTO> tableCalculateList = Optional.<List<CalculateTemplateTableCalculateApiDTO>>ofNullable((List<CalculateTemplateTableCalculateApiDTO>)((Map)this.tableCalculateMap.get(type)).get(tableId)).orElse(Lists.newArrayList());
    CalculateTemplateMetaApiDTO calculateTemplateMetaApiDTO = (CalculateTemplateMetaApiDTO)((Map)Optional.<Map>ofNullable(this.metaMap.get(type)).orElse(Maps.newHashMap())).get(calculateTemplateTableApiDTO.getMetaId());
    CalculateTemplateEdgeApiDTO calculateTemplateEdgeApiDTO = (CalculateTemplateEdgeApiDTO)((Map)Optional.<Map>ofNullable(this.edgeMap.get(type)).orElse(Maps.newHashMap())).get(calculateTemplateTableApiDTO.getEdgeId());
    CarbonFootprintParamConvertResult.CalculateStandardParam param = new CarbonFootprintParamConvertResult.CalculateStandardParam();
    if (Objects.nonNull(calculateTemplateEdgeApiDTO))
      param.setEdgeName(calculateTemplateEdgeApiDTO.getEdgeName()); 
    if (Objects.nonNull(calculateTemplateMetaApiDTO))
      param.setMetaName(calculateTemplateMetaApiDTO.getMetaName()); 
    param.setRelateTableId(calculateTemplateTableApiDTO.getRelateTableId());
    param.setEdgeId(calculateTemplateTableApiDTO.getEdgeId());
    param.setMetaId(calculateTemplateTableApiDTO.getMetaId());
    param.setTableId(calculateTemplateTableApiDTO.getTableId());
    param.setTableName(calculateTemplateTableApiDTO.getTableName());
    param.setCalculateSign(calculateTemplateTableApiDTO.getCalculateSign());
    param.setDecimalScale(calculateTemplateTableApiDTO.getDecimalScale());
    List<CarbonFootprintParamConvertResult.CalculateStandardTableFormula> formulaList = (List<CarbonFootprintParamConvertResult.CalculateStandardTableFormula>)tableCalculateList.stream().map(item -> {
          CarbonFootprintParamConvertResult.CalculateStandardTableFormula formula = (CarbonFootprintParamConvertResult.CalculateStandardTableFormula)BeanConverter.to(item, CarbonFootprintParamConvertResult.CalculateStandardTableFormula.class);
          formula.setCalculateFormula(item.getCalculateFormulaDisplay());
          formula.setCalculateFormulaExpress(item.getCalculateFormula());
          return formula;
        }).collect(Collectors.toList());
    param.setFormulaList(formulaList);
    return param;
  }
  
  private List<CarbonFootprintParamConvertResult.CalculateDataFillItem> handleItem(List<CarbonFootprintParamRequest.CalculateDataRow> dataRowList, Long tableId, String type) {
    return (List<CarbonFootprintParamConvertResult.CalculateDataFillItem>)dataRowList.stream().map(item -> {
          List<CarbonFootprintParamRequest.CalculateDataColumn> dataColumnList = (List<CarbonFootprintParamRequest.CalculateDataColumn>)item.getDataColumnList().stream().filter(()).collect(Collectors.toList());
          Map<String, List<CarbonFootprintParamRequest.CalculateDataColumn>> columnGroup = (Map<String, List<CarbonFootprintParamRequest.CalculateDataColumn>>)dataColumnList.stream().collect(Collectors.groupingBy(CarbonFootprintParamRequest.CalculateDataColumn::getColumnCode));
          columnGroup.forEach(());
          List<CarbonFootprintParamConvertResult.CalculateDataFillItemValue> itemValueList = (List<CarbonFootprintParamConvertResult.CalculateDataFillItemValue>)dataColumnList.stream().map(()).collect(Collectors.toList());
          long count = itemValueList.stream().filter(()).map(CarbonFootprintParamConvertResult.CalculateDataFillItemValue::getTableId).distinct().count();
          List<String> columnCodeList = (List<String>)itemValueList.stream().map(CarbonFootprintParamConvertResult.CalculateDataFillItemValue::getValueCode).collect(Collectors.toList());
          ThrowEnumUtils.isTure((count > 1L)).throwMessage((BaseExceptionEnum)ResultEnum.STANDARD_TABLE_COLUMN_CODE_ERROR, new Object[] { StringUtils.join((Object[])new List[] { columnCodeList }) });
          CalculateTemplateItemApiDTO calculateTemplateItemApiDTO = appendDefaultValue(itemValueList, tableId, type, item.getRowId());
          checkValue(itemValueList, tableId, type);
          CarbonFootprintParamConvertResult.CalculateDataFillItem fillItem = createItem(calculateTemplateItemApiDTO, itemValueList, tableId, type);
          fillItem.setRowId(item.getRowId());
          fillItem.setActivityDataType(item.getActivityDataType());
          fillItem.setEmissionFactorType(item.getEmissionFactorType());
          return fillItem;
        }).collect(Collectors.toList());
  }
  
  private void checkValue(List<CarbonFootprintParamConvertResult.CalculateDataFillItemValue> itemValueList, Long tableId, String type) {
    CalculateCarbonFootprintTableApiDTO calculateTemplateTableApiDTO = (CalculateCarbonFootprintTableApiDTO)((Map)this.tableMap.get(type)).get(tableId);
    List<CalculateTemplateFieldApiDTO> fieldList = (List<CalculateTemplateFieldApiDTO>)((Map)this.fieldMap.get(type)).get(tableId);
    if (Objects.isNull(calculateTemplateTableApiDTO.getTableId())) {
      Map<String, CalculateTemplateFieldApiDTO> thisTableFieldMap = (Map<String, CalculateTemplateFieldApiDTO>)fieldList.stream().filter(i -> CalculateItemValueTypeEnum.APPEND.name().equals(i.getFieldType())).collect(Collectors.toMap(CalculateTemplateFieldApiDTO::getCode, Function.identity()));
      itemValueList.forEach(itemValue -> ThrowEnumUtils.isTure(Objects.isNull(thisTableFieldMap.get(itemValue.getFieldCode()))).throwMessage((BaseExceptionEnum)ResultEnum.STANDARD_FIELD_CODE_NOT_EXIST, new Object[] { itemValue.getFieldCode() }));
      return;
    } 
    List<String> formulaFieldNames = (List<String>)((List)Optional.<List>ofNullable((List)((Map)this.tableCalculateMap.get(type)).get(tableId)).orElse(Lists.newArrayList())).stream().filter(item -> item.getCalculateFormula().contains("=")).map(formula -> formula.getCalculateFormula().substring(1, formula.getCalculateFormula().indexOf("=") - 1)).collect(Collectors.toList());
    Map<String, CarbonFootprintParamConvertResult.CalculateDataFillItemValue> valueMap = (Map<String, CarbonFootprintParamConvertResult.CalculateDataFillItemValue>)itemValueList.stream().collect(Collectors.toMap(CarbonFootprintParamConvertResult.CalculateDataFillItemValue::getFieldCode, Function.identity()));
    fieldList.forEach(field -> {
          if (CalculateItemValueTypeEnum.APPEND.name().equals(field.getFieldType()))
            return; 
          List<String> units = StringUtils.isNotBlank(field.getUnits()) ? (List<String>)Arrays.<String>stream(field.getUnits().trim().split(",")).collect(Collectors.toList()) : Lists.newArrayList();
          CarbonFootprintParamConvertResult.CalculateDataFillItemValue value = (CarbonFootprintParamConvertResult.CalculateDataFillItemValue)valueMap.get(field.getCode());
          value.setValue(value.getValue().trim());
          if (Objects.isNull(value) && formulaFieldNames.contains(field.getCode())) {
            value = generateItemValue(field);
            itemValueList.add(value);
            handleValueUnit(value, units);
            return;
          } 
          if (Objects.nonNull(value) && StringUtils.isBlank(value.getValue()) && formulaFieldNames.contains(field.getCode())) {
            handleValueUnit(value, units);
            return;
          } 
          ThrowEnumUtils.isTure((CalculateStandardFieldValueTypeEnum.CALCULATE.getCode().equals(field.getFieldValueType()) && (Objects.isNull(value) || Objects.isNull(value.getDependencies())))).throwMessage((BaseExceptionEnum)ResultEnum.STANDARD_TABLE_CALCULATE_FIELD_DEPENDENCE_IS_NULL, new Object[] { field.getCode() });
          if (!CalculateStandardFieldValueTypeEnum.CALCULATE.getCode().equals(field.getFieldValueType())) {
            ThrowEnumUtils.isTure((Objects.isNull(value) || StringUtils.isBlank(value.getValue()))).throwMessage((BaseExceptionEnum)ResultEnum.STANDARD_TABLE_ITEM_VALUE_IS_NULL, new Object[] { field.getCode() });
            if (CalculateItemValueTypeEnum.FACTOR.name().equals(field.getFieldType()) || CalculateItemValueTypeEnum.CONSUMPTION.name().equals(field.getFieldType()))
              ThrowEnumUtils.isTure(!NumberUtil.isNumber(value.getValue())).throwMessage((BaseExceptionEnum)ResultEnum.STANDARD_VALUE_NOT_NUMBER, new Object[] { value.getValue() }); 
          } 
          if (CalculateItemValueTypeEnum.ENERGY_CATEGORY.name().equals(field.getFieldType()))
            ThrowEnumUtils.isTure(!this.energyCategoryList.contains(value.getValue())).throwMessage((BaseExceptionEnum)ResultEnum.STANDARD_ENERGY_CATEGORY_VALUE_NOT_EXIST, new Object[] { value.getValue() }); 
          handleValueUnit(value, units);
        });
  }
  
  private CalculateTemplateItemApiDTO appendDefaultValue(List<CarbonFootprintParamConvertResult.CalculateDataFillItemValue> itemValueList, Long tableId, String type, Long rowId) {
    if (CollectionUtils.isEmpty(itemValueList))
      return null; 
    CalculateCarbonFootprintTableApiDTO table = (CalculateCarbonFootprintTableApiDTO)((Map)this.tableMap.get(type)).get(tableId);
    Long templateTableId = "TEMPLATE".equals(type) ? table.getTableId() : table.getTableId();
    if (Objects.isNull(templateTableId))
      return null; 
    List<CalculateTemplateItemApiDTO> templateItemList = getTemplateItemList(templateTableId);
    Map<String, String> dataItemNameMap = replaceSelectTreeItemValue(tableId, itemValueList, type, rowId);
    Map<String, CarbonFootprintParamConvertResult.CalculateDataFillItemValue> valueMap = (Map<String, CarbonFootprintParamConvertResult.CalculateDataFillItemValue>)itemValueList.stream().filter(i -> StringUtils.isNotBlank(i.getValueCode())).collect(Collectors.toMap(i -> StringUtils.defaultString(i.getValueCode(), i.getFieldCode()), Function.identity()));
    CalculateTemplateItemApiDTO calculateTemplateItemApiDTO = matchTemplateItem(templateItemList, dataItemNameMap);
    if (Objects.nonNull(calculateTemplateItemApiDTO)) {
      Map<String, String> fieldCodeMap = (Map<String, String>)((List)((Map)this.fieldMap.get(type)).get(tableId)).stream().collect(Collectors.toMap(CalculateTemplateFieldApiDTO::getFieldCode, CalculateTemplateFieldApiDTO::getCode));
      List<CalculateTemplateItemValueApiDTO> templateItemValueList = calculateTemplateItemApiDTO.getItemValueList();
      templateItemValueList.stream().filter(itemValue -> valueMap.containsKey(itemValue.getValueCode())).forEach(itemValue -> {
            CarbonFootprintParamConvertResult.CalculateDataFillItemValue fillValue = (CarbonFootprintParamConvertResult.CalculateDataFillItemValue)valueMap.get(itemValue.getValueCode());
            String fieldCode = Optional.<String>ofNullable((String)fieldCodeMap.get(itemValue.getValueCode())).orElse("");
            if (CalculateItemValueScalarTypeEnum.ENUM.getKey().equals(itemValue.getValueScalarType()))
              changeEnumKeyToValue(fillValue, itemValue, fieldCode); 
            ThrowEnumUtils.isTure(("RANGE".equals(itemValue.getRangeType()) && (Objects.isNull(fillValue) || StringUtils.isBlank(fillValue.getValue())))).throwMessage((BaseExceptionEnum)ResultEnum.STANDARD_TABLE_ITEM_VALUE_IS_NULL, new Object[] { fieldCode });
            if (Objects.isNull(fillValue)) {
              CarbonFootprintParamConvertResult.CalculateDataFillItemValue appendItemValue = new CarbonFootprintParamConvertResult.CalculateDataFillItemValue();
              appendItemValue.setValue(itemValue.getValue());
              appendItemValue.setOriginValue(itemValue.getValue());
              appendItemValue.setValueType(itemValue.getValueType());
              appendItemValue.setValueUnit(itemValue.getValueUnit());
              appendItemValue.setValueScalarType(itemValue.getValueScalarType());
              appendItemValue.setValueCode(itemValue.getValueCode());
              appendItemValue.setValueFillType(FillValueTypeEnum.VFT2.name());
              appendItemValue.setTableId(tableId);
              appendItemValue.setFieldCode(fieldCode);
              itemValueList.add(appendItemValue);
            } 
            if (Objects.nonNull(fillValue) && StringUtils.isBlank(fillValue.getValueUnit()))
              fillValue.setValueUnit(itemValue.getValueUnit()); 
            if (Objects.nonNull(fillValue) && StringUtils.isNotBlank(fillValue.getValue()) && StringUtils.isNotBlank(itemValue.getValue()) && FillValueTypeEnum.VFT2.name().equals(fillValue.getValueFillType()) && "RANGE".equals(itemValue.getRangeType())) {
              String[] values = itemValue.getValue().trim().split(",");
              BigDecimal value = new BigDecimal(fillValue.getValue());
              ThrowEnumUtils.isTure((value.compareTo(new BigDecimal(values[0])) < 0 || value.compareTo(new BigDecimal(values[1])) > 0)).throwMessage((BaseExceptionEnum)ResultEnum.PARAMS_TABLE_FIELD_RANGE_VALUE_ERROR, new Object[] { fillValue.getFieldCode() });
            } 
            if (Objects.nonNull(fillValue) && StringUtils.isNotBlank(fillValue.getValue()) && StringUtils.isNotBlank(itemValue.getValue()) && FillValueTypeEnum.VFT2.name().equals(fillValue.getValueFillType()) && !"RANGE".equals(itemValue.getRangeType()) && (CalculateItemValueScalarTypeEnum.DECIMAL.getKey().equals(itemValue.getValueScalarType()) || CalculateItemValueScalarTypeEnum.PERCENT.getKey().equals(itemValue.getValueScalarType())))
              ThrowEnumUtils.isTure(!fillValue.getValue().equals(itemValue.getValue())).throwMessage((BaseExceptionEnum)ResultEnum.PARAMS_TABLE_FIELD_DEFAULT_VALUE_ERROR, new Object[] { fillValue.getFieldCode() }); 
          });
    } 
    return calculateTemplateItemApiDTO;
  }
  
  private void changeEnumKeyToValue(CarbonFootprintParamConvertResult.CalculateDataFillItemValue fillValue, CalculateTemplateItemValueApiDTO templateValue, String code) {
    ThrowEnumUtils.isTure((Objects.isNull(fillValue) || StringUtils.isBlank(fillValue.getValue()))).throwMessage((BaseExceptionEnum)ResultEnum.STANDARD_TABLE_ITEM_VALUE_IS_NULL, new Object[] { code });
    if (!FillValueTypeEnum.VFT2.name().equals(fillValue.getValueFillType()))
      return; 
    if (NumberUtils.isParsable(fillValue.getValue()))
      return; 
    String keyName = this.factorEnumMap.get(fillValue.getValue());
    ThrowEnumUtils.isBlank(keyName).throwMessage((BaseExceptionEnum)ResultEnum.STANDARD_TABLE_ITEM_VALUE_ERROR, new Object[] { code });
    List<CalculateItemValueEnumDTO> list = JSON.parseArray(templateValue.getValue(), CalculateItemValueEnumDTO.class);
    Map<String, String> enumMap = (Map<String, String>)list.stream().collect(Collectors.toMap(CalculateItemValueEnumDTO::getLabel, CalculateItemValueEnumDTO::getValue));
    String enumValue = enumMap.get(keyName);
    ThrowEnumUtils.isBlank(enumValue).throwMessage((BaseExceptionEnum)ResultEnum.STANDARD_TABLE_ITEM_VALUE_ERROR, new Object[] { code });
    fillValue.setValue(enumValue);
    fillValue.setOriginValue(keyName);
  }
  
  private List<CalculateTemplateItemApiDTO> getTemplateItemList(Long tableId) {
    List<CalculateTemplateItemApiDTO> templateItemList = (List<CalculateTemplateItemApiDTO>)this.itemList.stream().filter(item -> tableId.equals(item.getTableId())).collect(Collectors.toList());
    Map<String, CalculateTemplateItemApiDTO> itemMap = (Map<String, CalculateTemplateItemApiDTO>)templateItemList.stream().filter(item -> Objects.isNull(item.getEnableDate())).collect(Collectors.toMap(CalculateTemplateItemApiDTO::getItemName, Function.identity(), (a, b) -> a));
    return Lists.newArrayList(itemMap.values());
  }
  
  private CarbonFootprintParamConvertResult.CalculateDataFillItem createItem(CalculateTemplateItemApiDTO calculateTemplateItemApiDTO, List<CarbonFootprintParamConvertResult.CalculateDataFillItemValue> itemValueList, Long tableId, String type) {
    CarbonFootprintParamConvertResult.CalculateDataFillItem item = new CarbonFootprintParamConvertResult.CalculateDataFillItem();
    item.setItemId(Long.valueOf(Objects.nonNull(calculateTemplateItemApiDTO) ? calculateTemplateItemApiDTO.getItemId().longValue() : IdWorker.getId()));
    item.setIsUserDefine(Integer.valueOf(Objects.nonNull(calculateTemplateItemApiDTO) ? 0 : 1));
    item.setItemName("");
    item.setItemValueList(itemValueList);
    List<CalculateTemplateFieldApiDTO> tableItemFieldList = findTableItemFieldList(tableId, type);
    Map<String, String> itemValueMap = (Map<String, String>)itemValueList.stream().filter(itemValue -> StringUtils.isNotBlank(itemValue.getOriginValue())).collect(Collectors.toMap(CarbonFootprintParamConvertResult.CalculateDataFillItemValue::getFieldCode, CarbonFootprintParamConvertResult.CalculateDataFillItemValue::getOriginValue));
    List<String> itemNames = (List<String>)tableItemFieldList.stream().map(itemValue -> (String)itemValueMap.get(itemValue.getCode())).collect(Collectors.toList());
    item.setItemName(Joiner.on('-').join((Iterable)itemNames));
    item.setTableId(tableId);
    return item;
  }
  
  private Map<String, String> replaceSelectTreeItemValue(Long tableId, List<CarbonFootprintParamConvertResult.CalculateDataFillItemValue> itemValueList, String type, Long rowId) {
    CalculateCarbonFootprintTableApiDTO table = (CalculateCarbonFootprintTableApiDTO)((Map)this.tableMap.get(type)).get(tableId);
    Long templateTableId = "TEMPLATE".equals(type) ? table.getTableId() : table.getTableId();
    Map<String, CarbonFootprintParamConvertResult.CalculateDataFillItemValue> itemValueMap = (Map<String, CarbonFootprintParamConvertResult.CalculateDataFillItemValue>)itemValueList.stream().collect(Collectors.toMap(CarbonFootprintParamConvertResult.CalculateDataFillItemValue::getFieldCode, Function.identity()));
    List<CalculateTemplateFieldApiDTO> itemFieldList = findTableItemFieldList(tableId, type);
    Map<String, String> dataItemValueMap = Maps.newHashMap();
    itemFieldList.forEach(field -> {
          CarbonFootprintParamConvertResult.CalculateDataFillItemValue itemValue = (CarbonFootprintParamConvertResult.CalculateDataFillItemValue)itemValueMap.get(field.getCode());
          ThrowEnumUtils.isNull(itemValue).throwMessage((BaseExceptionEnum)ResultEnum.STANDARD_FIELD_CODE_NOT_EXIST, new Object[] { field.getCode(), rowId });
          if ("SELECT_TREE".equals(field.getFieldControl())) {
            List<CalculateTemplateFieldItemApiDTO> templateFieldItemList = this.fieldItemMap.get(templateTableId);
            ThrowEnumUtils.isEmpty(templateFieldItemList).throwMessage((BaseExceptionEnum)ResultEnum.STANDARD_FIELD_ITEM_DATA_ERROR, new Object[0]);
            Optional<CalculateTemplateFieldItemApiDTO> fieldItemOptional = templateFieldItemList.stream().filter(()).findFirst();
            ThrowEnumUtils.isTure(!fieldItemOptional.isPresent()).throwMessage((BaseExceptionEnum)ResultEnum.STANDARD_ITEM_NAME_NOT_EXIST, new Object[] { itemValue.getValue() });
            itemValue.setValueDesc(itemValue.getValue());
            itemValue.setOriginValue(itemValue.getValue());
            itemValue.setValue(((CalculateTemplateFieldItemApiDTO)fieldItemOptional.get()).getFieldItemId().toString());
            itemValue.setValueFillType(null);
          } 
          dataItemValueMap.put(field.getFieldCode(), itemValue.getValue());
        });
    return dataItemValueMap;
  }
  
  private List<CalculateTemplateFieldApiDTO> findTableItemFieldList(Long tableId, String type) {
    List<CalculateTemplateFieldApiDTO> tableFieldList = (List<CalculateTemplateFieldApiDTO>)((Map)this.fieldMap.get(type)).get(tableId);
    return (List<CalculateTemplateFieldApiDTO>)tableFieldList.stream()
      .filter(field -> CalculateItemValueTypeEnum.CALCULATE_ITEM.name().equals(field.getFieldType()))
      .sorted(Comparator.comparing(CalculateTemplateFieldApiDTO::getSequenceNo))
      .collect(Collectors.toList());
  }
  
  private CalculateTemplateItemApiDTO matchTemplateItem(List<CalculateTemplateItemApiDTO> templateItemList, Map<String, String> valueMap) {
    Optional<CalculateTemplateItemApiDTO> templateItemOptional = templateItemList.stream().filter(item -> {
          List<CalculateTemplateItemValueApiDTO> itemValueList = item.getItemValueList();
          List<CalculateTemplateItemValueApiDTO> filterList = (List<CalculateTemplateItemValueApiDTO>)itemValueList.stream().filter(()).filter(()).collect(Collectors.toList());
          if (CollectionUtils.isEmpty(filterList))
            return false; 
          for (CalculateTemplateItemValueApiDTO value : filterList) {
            String s = (String)valueMap.get(value.getValueCode());
            if (!value.getValue().equals(s))
              return false; 
          } 
          return true;
        }).findFirst();
    return templateItemOptional.orElse(null);
  }
  
  private CarbonFootprintParamConvertResult.CalculateDataFillItemValue covertItemValue(CarbonFootprintParamRequest.CalculateDataColumn columnData, Long tableId, String type) {
    CalculateTemplateFieldApiDTO calculateTemplateFieldApiDTO = (CalculateTemplateFieldApiDTO)((Map)this.fieldCodeMap.get(type)).get(columnData.getColumnCode());
    if (CalculateItemValueTypeEnum.APPEND.name().equals(calculateTemplateFieldApiDTO.getFieldType())) {
      log.debug("通用字段，直接复制转换值返回");
      return generateAppendItemValue(columnData);
    } 
    ThrowEnumUtils.isTure((StringUtils.isBlank(columnData.getValue()) && 
        CollectionUtils.isEmpty(columnData.getDependencies())))
      .throwMessage((BaseExceptionEnum)ResultEnum.STANDARD_COLUMN_VALUE_AND_DEPENDENCIES_IS_NULL, new Object[] { columnData.getColumnCode() });
    ThrowEnumUtils.isNull(calculateTemplateFieldApiDTO).throwMessage((BaseExceptionEnum)ResultEnum.STANDARD_FIELD_CODE_NOT_EXIST, new Object[] { columnData.getColumnCode() });
    List<String> units = StringUtils.isNotBlank(calculateTemplateFieldApiDTO.getUnits()) ? (List<String>)Arrays.<String>stream(calculateTemplateFieldApiDTO.getUnits().trim().split(",")).collect(Collectors.toList()) : Lists.newArrayList();
    CarbonFootprintParamConvertResult.CalculateDataFillItemValue itemValue = generateItemValue(calculateTemplateFieldApiDTO);
    if (CalculateItemValueTypeEnum.CALCULATE_ITEM.name().equals(calculateTemplateFieldApiDTO.getFieldType()))
      itemValue.setValueDesc(columnData.getValue()); 
    itemValue.setDisplayValue(columnData.getDisplayValue());
    if (StringUtils.isNotBlank(columnData.getValueUnit())) {
      ThrowEnumUtils.isTure((StringUtils.isNotBlank(calculateTemplateFieldApiDTO.getUnits()) && units
          .stream().filter(unit -> columnData.getValueUnit().equals(unit)).count() < 1L))
        .throwMessage((BaseExceptionEnum)ResultEnum.STANDARD_TABLE_FIELD_VALUE_UNIT_CAN_NOT_EXIST, new Object[] { calculateTemplateFieldApiDTO.getCode(), columnData.getValueUnit(), units });
      itemValue.setValueUnit(columnData.getValueUnit());
    } 
    itemValue.setAggFieldType(calculateTemplateFieldApiDTO.getReportFieldType());
    itemValue.setValue(columnData.getValue());
    itemValue.setOriginValue(columnData.getValue());
    itemValue.setValueFillType(StringUtils.isBlank(columnData.getValueFillType()) ? FillValueTypeEnum.VFT1.name() : columnData.getValueFillType());
    if (CollectionUtils.isNotEmpty(columnData.getDependencies())) {
      String dependenceType = Objects.isNull(calculateTemplateFieldApiDTO.getEdgeId()) ? "TEMPLATE" : "TEMPLATE";
      itemValue.setDependencies(covertParam(columnData.getDependencies(), tableId, dependenceType));
      itemValue.getDependencies().getItemList().forEach(item -> item.setTemplateFieldId(calculateTemplateFieldApiDTO.getFieldId()));
    } 
    return itemValue;
  }
  
  CarbonFootprintParamConvertResult.CalculateDataFillItemValue generateAppendItemValue(CarbonFootprintParamRequest.CalculateDataColumn columnData) {
    CarbonFootprintParamConvertResult.CalculateDataFillItemValue item = (CarbonFootprintParamConvertResult.CalculateDataFillItemValue)BeanConverter.deepTo(columnData, CarbonFootprintParamConvertResult.CalculateDataFillItemValue.class);
    item.setFieldCode(columnData.getColumnCode());
    item.setAggFieldType(FootprintAggFieldTypeEnum.fromNullableCode(columnData.getColumnCode()));
    return item;
  }
  
  private void handleValueUnit(CarbonFootprintParamConvertResult.CalculateDataFillItemValue itemValue, List<String> units) {
    if (Objects.isNull(itemValue) || CollectionUtils.isEmpty(units))
      return; 
    if (units.size() == 1 && StringUtils.isBlank(itemValue.getValueUnit()))
      itemValue.setValueUnit(units.get(0)); 
    ThrowEnumUtils.isTure((units.size() > 1 && StringUtils.isBlank(itemValue.getValueUnit()))).throwMessage((BaseExceptionEnum)ResultEnum.STANDARD_TABLE_FIELD_VALUE_UNIT_IS_NULL, new Object[] { itemValue.getFieldCode() });
  }
  
  private CarbonFootprintParamConvertResult.CalculateDataFillItemValue generateItemValue(CalculateTemplateFieldApiDTO field) {
    CarbonFootprintParamConvertResult.CalculateDataFillItemValue itemValue = new CarbonFootprintParamConvertResult.CalculateDataFillItemValue();
    itemValue.setValueCode(field.getFieldCode());
    if (StringUtils.isNotBlank(field.getUnits()) && (field.getUnits().trim().split(",")).length == 1)
      itemValue.setValueUnit(field.getUnits()); 
    itemValue.setTableId(field.getTableId());
    itemValue.setValueScalarType(field.getFieldValueType());
    itemValue.setValueType(field.getFieldType());
    itemValue.setFieldCode(field.getCode());
    return itemValue;
  }
}
